# coding: utf-8

from datetime import timedelta
import random


def generator(start, interval=timedelta(days=1), **noise):
    """
    :param start: datetime from. <datetime>
    :param interval: increase interval. <timedelta>
    :param **noise: timedelta args(days, hours, minutes, seconds, microseconds). <int> keyword-arguments
    :return: iterator object
    """
    dt = start

    if noise:
        while True:
            _noise = {}
            for k, v in noise.items():
                if isinstance(v, tuple):
                    _noise[k] = random.randint(*v)
                else:
                    _noise[k] = random.randint(-v, v)

            yield dt + timedelta(**_noise)
            dt += interval
    else:
        while True:
            yield dt
            dt += interval


def range(start, end, interval=timedelta(days=1), callback=None, **noise):
    """
    :param start: datetime from. <datetime>
    :param end: datetime to. <datetime>
    :param interval: increase interval. <timedelta>
    :param callback: callback that each element. <function>
    :param **noise: timedelta args(days, hours, minutes, seconds, microseconds). <int> keyword-arguments
    :return: list
    """
    result = []
    for dt in generator(start, interval, **noise):
        if dt > end:
            return result
        else:
            if callback is None:
                result.append(dt)
            else:
                result.append(callback(dt))

